/***********************************************************************/
/*                                                                     */
/* RIGI Utilities --- Copyright (C) 1996-1999                          */
/*                    The Rigi Group, University of Victoria           */
/*                                                                     */
/* see the file license.terms for usage and copying policies           */
/*                                                                     */
/* Author: Johannes Martin (jmartin@csr.csc.uvic.ca)                   */
/*                                                                     */
/***********************************************************************/
#ifndef __RSFTUPLE_H__
#define __RSFTUPLE_H__

#include "uniqstr.h"

class RsfTuple {
protected:
  static void printElement(FILE* output, const char* element);
public:
  /*
   * Predefined UniqueStrings for some standard RSF key words.
   */
  static const UniqueString typeString;
  static const UniqueString locString;
  static const UniqueString multiarcString;

  /*
   * RSF tuple elements
   */
  UniqueString rsf[4];
  int line;
  int column;

  /*
   * Create a new RsfTuple.
   */
  RsfTuple(const UniqueString& elem0, const UniqueString& elem1,
	   const UniqueString& elem2, const UniqueString& elem3 = string(""),
	   int line = 0, int column = 0);

  /*
   * Parse a line of an RSF file and put its elements into an RsfTuple.
   * Remove the 4th element in the line, if preserve4Tuple is not set.
   * Errors and RSF comments will be printed to stderr. Check isValid()
   * to see whether the tuple was actually created.
   * If forgiving is true, incomplete tuples (less than 3 tuples) will be
   * accepted, and no error messages or comments will be printed.
   */
  RsfTuple(char *inputLine, unsigned long lineNo,
	   bool preserve4Tuple, bool forgiving = false);

  /*
   * Compare two tuples by comparing type and name. 'type' tuples come
   * first, 'loc' tuples last, in between, tuples are sorted by their
   * second element (third, first, fourth element as secondary sort keys).
   */
  bool operator< (const RsfTuple& other) const;

  /*
   * Print out an RsfTuple to output.
   * Remove the 4th element in the line, if preserve4Tuple is not set.
   * Return false if printf failes, otherwise true.
   */
  virtual bool print(FILE* output, bool preserve4Tuple = false) const;

  /*
   * Print out an RsfTuple to stdout.
   * Remove the 4th element in the line, if preserve4Tuple is not set.
   * Return false if printf failes, otherwise true.
   */
  bool print(bool preserve4Tuple = false) const {
    return print(stdout, preserve4Tuple);
  }

  /*
   * Check whether the tuple is valid.
   */
  bool isValid() const {
    return !rsf[0].empty();
  }
};

#endif
